######!/usr/bin/perl -w
#

package get_ifaces_info;
use strict;
use Data::Dumper;

# Definitions
my $boot_ifaces_path	= "/etc/network/interfaces";
my $cmd_ifconfig	= _test_cmd("ifconfig", "/sbin/ifconfig");

my $not_open_file	= "The file $boot_ifaces_path does not exist or can't be opened.\n";
my $error_ifconfig	= "There was an error during the execution of the command: $cmd_ifconfig.\n";

my $cmd_ethtool = _test_cmd("ethtool", "/usr/sbin/ethtool");

sub	_get_all_name 
{
    my $all;
    
    my @infos = qx!$cmd_ifconfig -a!;
    my $boot_ifc = _get_booted_ifaces();

    foreach my $var (@infos) {
	if ( $var =~ /^([\w:]+)\s+/) {
	    $all->{$1}= undef ;
	}
    }
    
    foreach my $key (keys %{$boot_ifc})
    {
	$all->{$key}= undef ;
    }
    
    return $all;
}

sub	_get_ifconf_all
{
    my $interfaces = shift;
    my $iface;
    my $mask;
    my $flg = 0;
    my @infos = qx!$cmd_ifconfig -a!;

    foreach my $var (@infos) {
	if ( $var =~ /^([\w:]+)\s+/) {
	    $iface = $1;
	    $interfaces->{$1} = {} if ( ! exists $interfaces->{$1});
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $interfaces->{$iface}{'inact'}{'ip'} = $1;	
	    $interfaces->{$iface}{'inact'}{'bcast'} = $2;
	    $interfaces->{$iface}{'inact'}{'mask'} = $3;
	}
    }
    return $interfaces;
}

sub	_get_ifconf
{
    my $interfaces = shift;
    my $iface;
    my @infos = qx!$cmd_ifconfig!;

    foreach my $var (@infos) {
	if ($var =~ /^([\w:]+)\s+/) {
	    $iface = $1;
	    $interfaces->{$1} = {} if ( ! exists $interfaces->{$1});
	    if ($var =~ /\s+HWaddr\s+([\w\d:]+)/) 
	    {
		$interfaces->{$iface}{'act'}{'macaddr'} = $1; 
	    }
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $interfaces->{$iface}{'act'}{'ip'} = $1;	
	    $interfaces->{$iface}{'act'}{'bcast'} = $2 ;	
	    $interfaces->{$iface}{'act'}{'mask'} = $3 ;
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $interfaces->{$iface}{'act'}{'ip'} = $1;	
	    $interfaces->{$iface}{'act'}{'mask'} = $2 ;	
	}
    }
    return $interfaces;
    
}

#Recuperer la listes des interfaces et de leurs caracteristiques au boot de la machine
sub	_get_booted_ifaces
{
    my $interfaces = shift;;
    my $if_name;
    my $tmp;
    
    if ( -f $boot_ifaces_path && -r _ )
    {
	open (FD, $boot_ifaces_path) or die "Error: $!\n";

	while (<FD>) {
	    next if ($_ =~ /^\s*\#/) ;
	    if ($_ =~ /^auto(.*)$/){
	        $tmp .= " ".$1." ";
	    }
	    elsif ( $_ =~ /^\s*iface\s+([\w\d:]*)\s+\w*\s+(\w*)/){
		$if_name = $1;
		$interfaces->{$if_name}{'boot'}{'mode'} = $2;
		$interfaces->{$if_name}{'boot'}{'boot'} = 'off';
	    }
	    elsif ( $_ =~ /\s+address\s+([\d\.]*)/ )
	    {
		$interfaces->{$if_name}{'boot'}{'ip'} = $1;
	    }
	    elsif ( $_ =~ /\s+netmask\s+([\d\.]*)/ )
	    {
		$interfaces->{$if_name}{'boot'}{'mask'} = $1 ;
	    }
	    elsif ( $_ =~ /\s+broadcast\s+([\d\.]*)/ )
	    {
		$interfaces->{$if_name}{'boot'}{'bcast'} = $1 ;
	    }
	    elsif ( $_ =~ /\s+gateway\s+([\d\.]*)/ )
	    {
		$interfaces->{$if_name}{'boot'}{'gw'} = $1 ;
	    }

	}

	foreach (split(" ",$tmp) ){
	    $interfaces->{$_}{'boot'}{'boot'} = 'on';
	}
	close (FD);
    }
    else{
	print $not_open_file;
	return 1;
    }

    return $interfaces;

}

sub	_got_link
{
    my $interfaces = shift;
    my @infos;
    
    foreach my $iface (sort keys %{$interfaces}){
	@infos = qx!$cmd_ethtool $iface 2>/dev/null !;
	$interfaces->{$iface}{'boot'}{"linked"} = "no";
	$interfaces->{$iface}{'act'}{"linked"} = "no";
	$interfaces->{$iface}{'inact'}{"linked"} = "no";
	foreach my $line (@infos) {
	    if ($line =~ /\s+detected:\s+(\w+)/){
		$interfaces->{$iface}{'boot'}{"linked"} = $1;
		$interfaces->{$iface}{'act'}{"linked"} = $1;
		$interfaces->{$iface}{'inact'}{"linked"} = $1;
	    } 
	}
    }
    return $interfaces;
}

sub	get_all_info
{
    my $interfaces;
    
    $interfaces = _get_booted_ifaces($interfaces);
    $interfaces = _get_ifconf($interfaces);
    $interfaces = _get_ifconf_all($interfaces);
    $interfaces = _got_link($interfaces);
    
    return $interfaces;
}

# Test sur le path et l'existence de la commande ifconfig
sub	_test_cmd
{
    my $cmd	= shift;
    my $name	= shift;
    my $func;
    
    chomp($func = `which $cmd`);
    
    if ( ! -e $func){
	$func = $name;
    }

    if ( ! -x $func){
	print "The command $cmd is not executable.\n";
	exit 0;
    }
    return $func;
}

1;
