#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## set_iface.pm for set network interface in 
## 
## ######### PROJECT DESCRIPTION : ###
## 
## 
## 
## ###################################
## 
## Made by Mickael
## Login   <root@intellique.com>
## 
## Started on  Thu Jan 17 16:41:11 2008 Mickael
## Last update Wed Feb 27 16:30:03 2008 Sardinha Mickael
## 
## ###################################
##

## RETURN FUNCTION: RETRUNCODE,RETURNMSG
## ex: 1,erreur or 0,ok

package set_iface;
use strict;
use get_ifaces_info;
use Data::Dumper;
use Net::IP qw(ip_is_ipv4);

# DEFINITIONS
my $config		= "/etc/network/interfaces";
my $ifconfig		= _test_cmd("ifconfig", "/sbin/ifconfig");
my $ifdown		= _test_cmd("ifdown", "/sbin/ifdown");
my $pump		= _test_cmd("pump", "/sbin/pump");

my $param = {
    'address'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'netmask'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'broadcast'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'network'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'gateway'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
};

sub     _test_cmd
{
    my $cmd     = shift;
    my $name    = shift;
    my $func;

    chomp($func = `which $cmd`);

    if ( ! -e $func){
        $func = $name;
    }

    if ( ! -x $func){
        print "$cmd : command does not exist\n";
        exit 0;
    }
    return $func;
}

 sub	_read_conf_file
{
    my $info;
    my $name;
    my $iface = shift;

    if ( -f $config && -r _ ) {
	open(FD_READ, "< $config") or die "Error: $!.\n";
	my @tab = <FD_READ>;
	close(FD_READ);
	
	foreach my $line (@tab){
	    if ( $line =~ /^(\#.*)/) {
		$info->{'commentaires'} .= $1."\n";
		next;
	    }
	    if ($line =~ /^auto\s+(.*[\w\d:])\s*$/){
		$info->{'auto'} = $1;
		next;
	    }
	    if ($line =~ /^iface\s+([\w+\d:]+)\s+inet\d?\s+(\w+)/){
		$name = $1;
		$info->{'inter'}{$name}{'mode'} = $2;
	    }
	    else {
		if ( $line =~ /^\s*(\w+)\s+(.*)/){
		    my $label = $1;
		    my $value = $2;
		    if ( exists $param->{$label} and ($value =~ /$param->{$label}/) )
		    {
			$info->{'inter'}{$name}{$label} = $value;
		    }
		}
		elsif ( $line =~ /^\s*(\#.*)/) {
		    $info->{'inter'}{$name}{'commentaires'} = $1;
		}
	    }
	}
	return $info;
    }
    else
    {
	return(1);
    }
}

sub	_write_conf_file
{
    my $hash_conf = shift;

    open(FD, "> $config") or return(1,"Error: $!.\n");    
    
    print FD $hash_conf->{'commentaires'}."\n" if ( exists $hash_conf->{'commentaires'} );
  
    print FD "auto ".$hash_conf->{'auto'}."\n\n";
    
    if (exists $hash_conf->{'inter'}{'lo'} and $hash_conf->{'inter'}{'lo'}{'mode'}) {
	print FD "iface lo inet ".$hash_conf->{'inter'}{'lo'}{'mode'}."\n\n";
	delete $hash_conf->{'inter'}{'lo'};
    }

    foreach my $key (sort  keys %{$hash_conf->{'inter'}}) {
	print FD "iface $key inet ".$hash_conf->{'inter'}{$key}{'mode'}."\n";
	print FD "\taddress ".$hash_conf->{'inter'}{$key}{'address'}."\n"	if ( $hash_conf->{'inter'}{$key}{'address'} );
	print FD "\tbroadcast ".$hash_conf->{'inter'}{$key}{'broadcast'}."\n"	if ( $hash_conf->{'inter'}{$key}{'broadcast'} );
	print FD "\tnetmask ".$hash_conf->{'inter'}{$key}{'netmask'}."\n"	if ( $hash_conf->{'inter'}{$key}{'netmask'} );
	print FD "\tnetwork ".$hash_conf->{'inter'}{$key}{'network'}."\n"	if ( $hash_conf->{'inter'}{$key}{'network'} );
	print FD "\tgateway ".$hash_conf->{'inter'}{$key}{'gateway'}."\n"	if ( $hash_conf->{'inter'}{$key}{'gateway'} );
	print FD "\t".$hash_conf->{'inter'}{$key}{'commentaires'}."\n"	if ( $hash_conf->{'inter'}{$key}{'commentaires'} );
	print FD "\n";
    }
    close(FD);
    
    return (0,"ok");
}

sub	set_interface
{
    my $hash= shift ;
    my $er;
    my $msg;
    my $msg2;
     foreach my $iface (keys %{$hash}) {
	foreach my $info (keys %{$hash->{$iface}}) {
	    if ($info =~ /boot/) {
		($er, $msg2) = _do_changes($iface,$hash->{$iface}{$info});
	    }
	    elsif ($info =~ /act/) {
		($er, $msg) = active_iface($iface, $hash->{$iface}{$info});
	    }
	}
    }
    return ($er, $msg."#".$msg2);
}

sub	_do_changes
{
     
    my ($iface,$info) = @_;
    my $hash_conf_file;
    my $ret;
    my $mg;
    my $field;
    my @tab;

    foreach my $k (keys %{$info}){
	push @tab, $k;
    }
    return (1,"No boot data found\n") if ( ! scalar(@tab));
    
    my ($er, @champ) = check_ip($info) unless ($info->{'mode'} eq "dhcp");
    
    if($er){
	$field = join(', ',@champ);
	return(1,"$field is/are not valid for $iface in boot configuration");
    }
    if ( ($hash_conf_file = _read_conf_file($iface)) == 1)
    { return (1,"Error in opening /etc/network/interfaces\n");}

    if (exists $info->{'boot'} )
    {
	if ($info->{'boot'} eq "on" )
	{
	    if ( $hash_conf_file->{'auto'} !~ /$iface[\s\$]/ ) {
		$hash_conf_file->{'auto'} .= " ".$iface;
	    }
	}
	elsif ($info->{'boot'} eq "off" )  {
	    if ( $hash_conf_file->{'auto'} =~ /(.*)\s*$iface\s*$/) {
		$hash_conf_file->{'auto'} = $1;
	    }
	    elsif ($hash_conf_file->{'auto'} =~ /(.*)\s*$iface\s+(.*)$/)
	    {
		$hash_conf_file->{'auto'} = $1." ".$2;
	    }
	}
	else{
	    return (1,"\"boot\" parameter is badly set");
	}
    }
    if ( exists $info->{'mode'} && $info->{'mode'} eq "dhcp")
    {
	delete $hash_conf_file->{'inter'}{$iface} if exists $hash_conf_file->{'inter'}{$iface};
	$hash_conf_file->{'inter'}{$iface}{'mode'} = "dhcp";
    }
    else {
	$hash_conf_file->{'inter'}{$iface}{'mode'}	= $info->{'mode'} if exists $info->{'mode'};
	$hash_conf_file->{'inter'}{$iface}{'address'}	= $info->{'ip'} if exists $info->{'ip'};
	$hash_conf_file->{'inter'}{$iface}{'netmask'}	= $info->{'mask'} if exists $info->{'mask'};
	$hash_conf_file->{'inter'}{$iface}{'broadcast'}	= $info->{'bcast'} if exists $info->{'bcast'};
	$hash_conf_file->{'inter'}{$iface}{'gateway'}	= $info->{'gw'} if exists $info->{'gw'};
    }
    
    ($ret, $mg) =_write_conf_file($hash_conf_file);
    
    return (1, $mg) if ($ret == 1);

    return(0," $iface: Configuration written in /etc/network/interfaces\n");
}

sub	active_iface
{
    my ($iface,$info) = @_;
    my $toto;
    my $field;
    my @tab;
    my $act_info = get_ifaces_info::_get_ifconf($toto);

    foreach my $k (keys %{$info}){
	push @tab, $k;
    }
    return (1,"No data found\n") if ( ! scalar(@tab));
    
    foreach my $var (keys %{$act_info}) {
	$info->{'ip'} = $act_info->{$var}{'act'}{'ip'} if ! defined $info->{'ip'};
	$info->{'mask'} = $act_info->{$var}{'act'}{'mask'} if ! defined $info->{'mask'};
    }
    my ($er, @champ) = check_ip($info);

    if($er){
	$field = join(', ',@champ);
	return(1,"$field is/are not valid for $iface in active configuration");
    }
    
    if (defined $info->{'new'} && $info->{'new'} != 0) {
	
	if ($info->{'mode'} eq "dhcp" ){
	    return(1,"Error: cannot create an interface in DHCP mode");
	}
    }
    
    if ( defined $info->{'mode'} && $info->{'mode'} eq "dhcp" ) {

	eval { qx!$pump -i $iface!;};
	
	if ($?) {
	    return(0,"Error during activation of $iface in DHCP mode. Operations aborted");
	}
	return(0, "Interface $iface configuration has been modified");
    }
    elsif ( ! defined $info->{'ip'} ||  ! defined $info->{'mask'}){
	return (1,"Error: Ip and Mask are required");
    }

    eval {qx!$ifconfig $iface inet $info->{'ip'} netmask $info->{'mask'} up !;};
    if ($?) {
	return(1, "Error during activation of $iface");
    }
    
    return(0, "Interface $iface configuration has been modified");
}

sub	check_ip
{
    my $hash = shift;
    my @erno_var;

    foreach my $var (keys %{$hash} )
    {
       	if ($var =~ /(^ga|^ip|^mask)/){
	    if (!$hash->{$var} or (! ip_is_ipv4($hash->{$var})))
	    {
		push(@erno_var, $var);
	    }
	}
    }
    
    if (defined $erno_var[0] ){
	return (1,@erno_var);
    }
    else{
	return(0,0);
    }
}


1;
